package net.sf.corn.gate;

import java.io.Serializable;
import java.util.Set;

import net.sf.corn.gate.IDataStage.Scope;

/**
 * A Service performs its operations inside a context that offers information about the
 * environment such as client, protocol, security. The context is also the access
 * point to data stages that services can save data. Since all service calls 
 * made through {@link CallDelegator}, services can access the active
 * client context information from {@link CallDelegator#getActiveClientContext()}
 * method.
 *</p>Clients may send data to services other than defined parameters of service operations. Those kind 
 *of data can be placed into the request header and can be accessed through the context. Also services can send
 *additional data as their response beside their operation returned data. That additional data 
 *can be placed to response headers by using the context again. A typical sample implementation of this
 *HTTP protocol specific rpc protocols in which client can put data to HTTP request headers and service can 
 *can place data into HTTP response headers.   
 *    
 * @author Serhat Dirik
 * 
 */
public interface IClientContext extends Serializable {

	/**
	 * Gives authenticated client user information
	 * 
	 * @return user name or null there is no authenticated user exist
	 */
	public String getClientUser();

	/**
	 * Checks if the authenticated client has the given role
	 * 
	 * @param role
	 *            role name
	 * @return true if user belongs to given role
	 */
	public boolean isUserInRole(String role);

	/**
	 * Gives the requested {@link IDataStage} that the client is already bounded by the underlying
	 * platform
	 * @return The requested data stage
	 */
	public IDataStage getDataStage(Scope type);

	/**
	 * Returns the communication protocol name
	 * 
	 * @return protocol name
	 */
	public String getProtocolName();

	/**
	 * Returns the client network address . For example it returns client's ip
	 * address for the Internet Protocol
	 * 
	 * @return Client network address
	 */
	public String getClientNetworkAddr();

	/**
	 * Returns the communication port if there is any. For the Internet Protocol
	 * (IP), source port of the client or last proxy that sent the request.
	 * 
	 * @return Communication Port
	 */
	public int getClientCommunicationPort();

	/**
	 * Returns the fully qualified host name of the client
	 * 
	 * @return host name of the client
	 */
	public String getClientHostName();

	/**
	 * Tells if client is in the same JVM
	 * 
	 * @return true if the client is a local one
	 */
	public boolean isLocal();

	/**
	 * Returns the host name of the server to which the request was sent.
	 */
	public String getServerName();

	/**
	 * Returns the port number to which the request was sent
	 */
	public int getServerPort();

	/**
	 * Returns the Internet Protocol (IP) address of the interface on which the
	 * request was received.
	 */
	public String getServerNetworkAddr();

	/**
	 * Returns a boolean indicating whether this request was made using a secure
	 * channel, such as HTTPS
	 * 
	 * @return true if the protocol is secure
	 */
	public boolean isProtocolSecure();

	/**
	 * Retrieves a value that previously saved with the given key into the response header 
	 * @param key The key that paired with a value in the response header
	 * @return The value that paired with the given key
	 */
	public Object getFromResponseHeader(String key);

	/**
	 * Gives set of the respose header keys
	 * @return set of key values 
	 */
	public Set<String> responseHeaderKeySet();

	/**
	 * Adds a key,value pair to the response header
	 * @param key key of to be mapped pair 
	 * @param value the object value
	 * @return returns the previously saved value with the given key  
	 */
	public Object addToResponseHeader(String key, Object value);
	
	/**
	 * Removes the value that mapped with the given key from the response header 
	 * @param key the of to be removed value
	 * @return previously saved value 
	 */
	public Object removeFromResponseHeader(String key);

	/**
	 * Tells if the questioned key is exist in the response header
	 * @param key the key to be checked 
	 * @return true if exist
	 */
	public boolean containsResponseHeaderKey(String key);

	/**
	 * Gives the requested value from the response header
	 * @param key the key of the mapped value
	 * @return The previously saved value or null 
	 */
	public Object getFromRequestHeader(String key);

	/**
	 * Gives the set of response header keys
	 * @return set of response header keys
	 */
	public Set<String> requestHeaderKeySet();

}
