package net.sf.corn.gate;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 * Service definition meta data 
 * 
 * @author Serhat Dirik
 */
public class ServiceDefinition implements Cloneable, Serializable {

	private static final long serialVersionUID = 145174043992884004L;
	
	/**
	 * Defines the service scope
	 * @author Serhat Dirik
	 */
	public enum ServiceScope {
		/**
		 * Gate uses the same service implementation object as singleton for all requests
		 */
		SINGLETON, 
		/**
		 * A service implementation object is initiated and used per client session
		 */
		SESSION, 
		/**
		 * Always a new service implementation instance is created for each request
		 */
		REQUEST;
	}

	

	private String name;
	private String locator = ILocator.DEFAULT_LOCATOR_NAME;
	private String componentTarget = null;
	private ServiceScope scope = ServiceScope.REQUEST;
	private boolean lazy = true;
	private String description = null;
	private boolean authenticationRequiredToInspect = false;
	private boolean authenticationRequiredToUse = false;
	private boolean secureProtocolRequiredToInspect = false;
	private boolean secureProtocolRequiredToUse = false;
	private boolean remoteAccessEnabled = true;
	private CopyOnWriteArrayList<String> authorizedRolesToInspect = new CopyOnWriteArrayList<String>();
	private CopyOnWriteArrayList<String> authorizedRolesToUse = new CopyOnWriteArrayList<String>();
	private List<OperationDefinition> operations = new CopyOnWriteArrayList<OperationDefinition>();
	protected Map<String, String> properties = new ConcurrentHashMap<String, String>();
	private Class<?> annotatedInterface = null;
	/**
	 * Defines a new service definition object by its name and component target
	 * @param name name of the service
	 * @param componentTarget name of the component target
	 */
	public ServiceDefinition(String name, String componentTarget) {
		if (name == null || name.trim().length() == 0) {
			throw new IllegalArgumentException(
					"Name parameter is null or empty");
		}
		if (componentTarget == null || componentTarget.trim().length() == 0) {
			throw new IllegalArgumentException(
					"componentTarget parameter is null or empty");
		}
		this.name = name;
		this.componentTarget = componentTarget;
	}

	ServiceDefinition(Class<?> clazz) {
		if (clazz == null) {
			throw new IllegalArgumentException("null class requested!");
		}
		if (!clazz.isAnnotationPresent(GateService.class)) {
			throw new IllegalArgumentException(
					"Only @GateService anootated classes can be requested!");
		}

		boolean isServiceDefInterface = clazz.isInterface();
		GateService serviceDef = clazz.getAnnotation(GateService.class);

		if (!isServiceDefInterface) {
			throw new IllegalArgumentException(
					"Only interfaces can be annotated with @GateService.Please check "
							+ clazz.getCanonicalName());
		}

		if (serviceDef.componentTarget() == null) {
			throw new IllegalArgumentException(
					"ComponentTarget is not specified in service definition!Please check "
							+ clazz.getCanonicalName());
		}
		if (serviceDef.name() == null) {
			throw new IllegalArgumentException(
					"Name is not specified in service definition!Please check "
							+ clazz.getCanonicalName());
		}

		this.annotatedInterface = clazz;
		this.name = serviceDef.name() == null
				|| serviceDef.name().trim().length() == 0 ? null : serviceDef
				.name();
		this.locator = serviceDef.locator() == null
				|| serviceDef.locator().trim().length() == 0 ? null
				: serviceDef.locator();
		this.componentTarget = serviceDef.componentTarget() == null
				|| serviceDef.componentTarget().trim().length() == 0 ? null
				: serviceDef.componentTarget();
		this.scope = serviceDef.scope();
		this.lazy = serviceDef.lazy();
		this.description = serviceDef.description().trim().length() == 0 ? null
				: serviceDef.description();
		this.authenticationRequiredToInspect = serviceDef
				.authenticationRequiredToInspect();
		this.authenticationRequiredToUse = serviceDef
				.authenticationRequiredToInspect();
		this.secureProtocolRequiredToInspect = serviceDef
				.secureProtocolRequiredToInspect();
		this.secureProtocolRequiredToUse = serviceDef
				.secureProtocolRequiredToUse();
		this.remoteAccessEnabled = serviceDef.remoteAccessEnabled();
		if (serviceDef.authorizedRolesToInspect().length > 0) {
			for (String role : serviceDef.authorizedRolesToInspect()) {
				this.addAuthorizedRoleToInspect(role);
			}
		}
		if (serviceDef.authorizedRolesToUse().length > 0) {
			for (String role : serviceDef.authorizedRolesToUse()) {
				this.addAuthorizedRoleToUse(role);
			}
		}

		if (serviceDef.properties().length > 0) {
			for (StringPair pair : serviceDef.properties()) {
				this.addProperty(pair.key(), pair.value());
			}
		}
	}

	private ServiceDefinition(ServiceDefinition serviceDef) {
		if (serviceDef == null) {
			throw new IllegalArgumentException(
					"Service definition parameter is null or empty");
		}
		this.name = serviceDef.getName();
		this.setAuthenticationRequiredToInspect(serviceDef
				.isAuthenticationRequiredToInspect());
		this.setAuthenticationRequiredToUse(serviceDef
				.isAuthenticationRequiredToUse());
		this.setComponentTarget(serviceDef.getComponentTarget());
		this.setLazy(serviceDef.isLazy());
		this.setServiceScope(serviceDef.getServiceScope());
		this.setLocator(serviceDef.getLocator());
		this.setDescription(serviceDef.getDescription());
		this.setSecureProtocolRequiredToInspect(serviceDef
				.isSecureProtocolRequiredToInspect());
		this.setSecureProtocolRequiredToUse(serviceDef
				.isSecureProtocolRequiredToUse());
		this.setRemoteAccessEnabled(serviceDef.isRemoteAccessEnabled());
		for (String role : serviceDef.getAuthorizedRolesToInspect()) {
			this.addAuthorizedRoleToInspect(role);
		}
		for (String role : serviceDef.getAuthorizedRolesToUse()) {
			this.addAuthorizedRoleToUse(role);
		}

		for (OperationDefinition op : serviceDef.getOperations()) {
			this.addOperation(op);
		}
		for (String key : serviceDef.getPropertyKeys()) {
			this.addProperty(key, serviceDef.getProperty(key));
		}
	}

	/**
	 * Gives the list of extended properties 
	 * @return set of extended properties keys
	 */
	public Set<String> getPropertyKeys() {
		Set<String> keySet = new HashSet<String>();
		keySet.addAll(properties.keySet());
		return keySet;
	}

	/**
	 * Gives the extended property value
	 * @param propertyKey key of the requested extended property
	 * @return Value of the extended property
	 */
	public String getProperty(String propertyKey) {
		return properties.get(propertyKey);
	}

	/**
	 * Tells if the requested property exist
	 * @return true if the property exist 
	 */
	public boolean containsProperty(String propertyKey) {
		return properties.containsKey(propertyKey);
	}
	/**
	 * Adds a new extended property to the service definition
	 * @param propertyKey key of the extended property
	 * @param propertyValue the value of the extended property 
	 * @return returns the previous value or null for the first time set
	 */
 	public String addProperty(String propertyKey, String propertyValue) {
		return properties.put(propertyKey, propertyValue);
	}

 	/**
 	 * Name of the service 
 	 * @return name of the service
 	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the service name
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;
	}

	public String getLocator() {
		return locator;
	}
	/**
	 * Sets the service's locator 
	 * @param locator name of the locator
	 */
	public void setLocator(String locator) {
		this.locator = locator;
	}

	/**
	 * Gives the component target name
	 * @return component target name
	 */
	public String getComponentTarget() {
		return componentTarget;
	}

	public void setComponentTarget(String componentTarget) {
		this.componentTarget = componentTarget;
	}
	/**
	 * Scope of the service
	 * @return scope
	 */
	public ServiceScope getServiceScope() {
		return scope;
	}
	/**
	 * Sets the service scope
	 * @param scope 
	 */
	public void setServiceScope(ServiceScope scope) {
		if (scope == null) {
			throw new IllegalArgumentException("scope cannot be null");
		}
		this.scope = scope;
	}
	/**
	 * Tells if the service instances initialized lazily or at the registration time
	 * @return true if the service is lazily initialized
	 */
	public boolean isLazy() {
		return lazy;
	}
	/**
	 * Sets the lazy initialization strategy of the service
	 * @param lazy true if lazy initialization needed
	 */
	public void setLazy(boolean lazy) {
		this.lazy = lazy;
	}
	/**
	 * Gives the service description
	 * @return service description
	 */
	public String getDescription() {
		return description;
	}
	/**
	 * Sets the service description
	 * @param description
	 */
	public void setDescription(String description) {
		this.description = description;
	}
	public boolean isAuthenticationRequiredToInspect() {
		return authorizedRolesToInspect.size() > 0
				|| authenticationRequiredToInspect;
	}

	public void setAuthenticationRequiredToInspect(
			boolean authenticationRequiredToInspect) {
		this.authenticationRequiredToInspect = authenticationRequiredToInspect;
	}

	public void setAuthenticationRequiredToUse(
			boolean authenticationRequiredToUse) {
		this.authenticationRequiredToUse = authenticationRequiredToUse;
	}

	public boolean isAuthenticationRequiredToUse() {
		return authorizedRolesToUse.size() > 0 || authenticationRequiredToUse;
	}

	public boolean isSecureProtocolRequiredToInspect() {
		return secureProtocolRequiredToInspect;
	}

	public void setSecureProtocolRequiredToInspect(
			boolean secureProtocolRequiredToInspect) {
		this.secureProtocolRequiredToInspect = secureProtocolRequiredToInspect;
	}

	public boolean isSecureProtocolRequiredToUse() {
		return secureProtocolRequiredToUse;
	}

	public void setSecureProtocolRequiredToUse(
			boolean secureProtocolRequiredToUse) {
		this.secureProtocolRequiredToUse = secureProtocolRequiredToUse;
	}

	public boolean isRemoteAccessEnabled() {
		return remoteAccessEnabled;
	}

	public void setRemoteAccessEnabled(boolean remoteAccessEnabled) {
		this.remoteAccessEnabled = remoteAccessEnabled;
	}

	public List<String> getAuthorizedRolesToInspect() {
		return Collections.unmodifiableList(authorizedRolesToInspect);
	}

	public boolean addAuthorizedRoleToInspect(String role) {
		if (role == null || role.trim().length() == 0) {
			throw new IllegalArgumentException(
					"Role name parameter is null or empty");
		}
		if (authorizedRolesToInspect.contains(role.trim())) {
			return true;
		}
		return authorizedRolesToInspect.add(role.trim());
	}

	public boolean removeAuthorizedRoleToInspect(String role) {
		if (role == null || role.trim().length() == 0) {
			throw new IllegalArgumentException(
					"Role name parameter is null or empty");
		}
		if (authorizedRolesToInspect.contains(role.trim())) {
			return authorizedRolesToInspect.remove(role.trim());
		}
		return true;
	}

	public Class<?> getAnnotatedInterface() {
		return annotatedInterface;
	}

	public List<String> getAuthorizedRolesToUse() {
		return Collections.unmodifiableList(authorizedRolesToUse);
	}

	public boolean addAuthorizedRoleToUse(String role) {
		if (role == null || role.trim().length() == 0) {
			throw new IllegalArgumentException(
					"Role name parameter is null or empty");
		}
		if (authorizedRolesToUse.contains(role.trim())) {
			return true;
		}
		return authorizedRolesToUse.add(role.trim());
	}

	public boolean removeAuthorizedRoleToUse(String role) {
		if (role == null || role.trim().length() == 0) {
			throw new IllegalArgumentException(
					"Role name parameter is null or empty");
		}
		if (authorizedRolesToUse.contains(role.trim())) {
			return authorizedRolesToUse.remove(role.trim());
		}
		return true;
	}

	public Collection<OperationDefinition> getOperations() {
		return Collections.unmodifiableList(operations);
	}

	public OperationDefinition getOperation(String operationName) {
		for (OperationDefinition operation : operations) {
			if (operation.getName().equals(operationName.trim())) {
				return operation;
			}
		}
		return null;
	}

	public boolean addOperation(OperationDefinition operation) {
		if (operation == null ) {
			throw new IllegalArgumentException(
					"operation parameter is null or not well defined!");
		}
		if (this.operations.contains(operation)) {
			return true;
		}
		return this.operations.add(operation);
	}

	public boolean removeOperation(OperationDefinition operation) {
		if (operation == null ) {
			throw new IllegalArgumentException(
					"operation parameter is null or not well defined!");
		}
		if (this.operations.contains(operation)) {
			return this.operations.remove(operation);
		}
		return true;
	}

	@Override
	public ServiceDefinition clone() {
		ServiceDefinition clone = new ServiceDefinition(this);
		return clone;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("<Service> ").append("Name:").append(name);
		if (getLocator() != null) {
			sb.append(",Locator:").append(getLocator());
		}
		sb.append(", componentTarget:").append(getComponentTarget());
		return sb.toString();
	}
  
}
